package de.hdm_stuttgart.mi.exam;

import java.lang.annotation.Annotation;
import java.util.List;
import java.util.Vector;

import org.junit.rules.TestWatcher;
import org.junit.runner.Description;

public class ResultCollector extends TestWatcher {

	public final List<TestResult> failedTests = new Vector<TestResult>(),
			successfulTests = new Vector<TestResult>();

	@Override
	protected void failed(final Throwable e, final Description description) {
		failedTests.add(new TestResult(description.getDisplayName(),
				getMarking(description), false));
	}

	@Override
	protected void succeeded(final Description description) {
		successfulTests.add(new TestResult(description.getDisplayName(),
				getMarking(description), true));
	}

	private static Marking getMarking(final Description description) {

		final Marking marking = description.getAnnotation(Marking.class);
		if (null == marking) { // Default values: 1 point
			return new Marking() {
				
				@Override
				public Class<? extends Annotation> annotationType() {
					return Marking.class;
				}
				
				@Override
				public int points() {
					return 1;
				}
			};

		}
		return marking;
	}

	@Override
	public String toString() {

		int reachablePoints = 0, reachedPoints = 0;

		for (final TestResult t : successfulTests) {
			reachablePoints += t.marking.points();
			reachedPoints += t.marking.points();
		}
		for (final TestResult t : failedTests) {
			reachablePoints += t.marking.points();
		}

		final StringBuffer buffer = new StringBuffer();

		buffer.append("You have reached " + reachedPoints + " of "
				+ reachablePoints + " possible points in "
				+ successfulTests.size() + " successful tests\n");
		if (1 == failedTests.size()) {
			buffer.append("The following test still fails:\n");
			buffer.append("  - " + failedTests.get(0).testMethodDisplayName
					+ "\n");
		} else if (1 < failedTests.size()) {
			buffer.append("The following " + failedTests.size()
					+ " tests still fail:\n");
			for (final TestResult t : failedTests) {
				buffer.append("  - " + t.testMethodDisplayName + "\n");
			}
		}
		return buffer.toString();
	}
}