#!/usr/bin/perl

use strict;

my $reachedPoints = 0;
my $totalPoints = 0;

my $reportFile = "report.log";


my %negatives = (

    "hasReference", ["Invalid content was found starting with element 'sourcecode'.", 1]
    ,"idNoSpace", ["'foo bar' is not a valid value for ", 1]
    ,"multipleReference", [" with value 'D#' not found for identity constraint", 1]
    ,"singleReference", [" with value 'simula' not found for identity constraint", 1]
    ,"uniqueId", ['Duplicate key value [foo] declared for identity constraint', 1]
    ,"uniqueLanguage", ['Duplicate key value [fortran] declared for identity constraint', 2]
    ,"uniqueMultipleLanguageRef", ["Duplicate key value [Fortran] declared for identity constraint ", 1]

    );

my %positives = (
    "full", 2
    );

system("rm -f  $reportFile");

system("xmlparse  ../Schema/scripts.xsd > scripts.xsd.log 2>&1");
my $exitValue  = $? >> 8;

if (0 != $exitValue) {
    &append ("File scripts.xsd is not well-formed, exiting", $reportFile);
    exit -1;
}


foreach my $negTest (sort keys(%negatives)) {
    my $localPoints = $negatives{$negTest}[1];
    $totalPoints += $localPoints;

    my $xmlInputValidFile = $negTest . ".xml";
    my $xmlInputValidLogFile =  $xmlInputValidFile . ".log";
    system("xmlparse  -V -s $xmlInputValidFile > $xmlInputValidLogFile 2>&1");
    my $exitValue  = $? >> 8;
    
    if (0 ==  $exitValue) {# Check for invalidness
	my $xmlInputInvalidFile = $negTest . "_err.xml";
	my $xmlInputInvalidLogFile =  $xmlInputInvalidFile . ".log";

	&append ("Testing invalid file $xmlInputInvalidFile", $reportFile);
	system("xmlparse  -V -s $xmlInputInvalidFile > $xmlInputInvalidLogFile 2>&1");
	my $exitValue  = $? >> 8;
	if (0 !=  $exitValue) {
	    &append("Invalid file $xmlInputInvalidFile being flagged as invalid, adding $localPoints point(s)", $reportFile);
	    $reachedPoints += $localPoints;
	} else {
	    &append("Invalid file $xmlInputInvalidFile not being flagged as invalid, missing $localPoints point(s)", $reportFile);
	}
    } else {# Valid file declared as invalid
	 &append("Valid file $xmlInputValidFile flagged as invalid, missing $localPoints point(s)", $reportFile);
    }

    &append ("____________________________________________________", $reportFile);
} 

foreach my $posTest  (sort keys(%positives)) {
    my $localPoints = $positives{$posTest};
    my $xmlInputFile = $posTest . ".xml";
    $totalPoints += $localPoints;
    
    &append("Testing valid file $xmlInputFile", $reportFile);
    system("xmlparse  -V -s $xmlInputFile >> $reportFile 2>> $reportFile");
    my $exitValue  = $? >> 8;
    if (0 ==  $exitValue) {
	
	&append("Validity confirmed expectedly, adding $localPoints point(s)", $reportFile);
	$reachedPoints += $localPoints;
    } else {
	&append("Valid File $xmlInputFile flagged as invalid, missing $localPoints point(s)",  $reportFile);
}
    &append ("____________________________________________________", $reportFile);
} 

system ("echo Summary: $reachedPoints out of $totalPoints points have been reached >> $reportFile");

exit $reachedPoints;


sub append {
    my ($text, $filename) = @_;
    open(my $fh, '>>',  $filename);
    say $fh $text;
    close $fh;
}




#end
